#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Copyright 2019 Pintaudi Giorgio

# pylint: disable=missing-docstring
# pylint: disable=too-few-public-methods
# pylint: disable=too-many-instance-attributes

import json
import sys
from copy import deepcopy
from functools import partial

try:
    # for Python2
    # noinspection PyPep8Naming,PyUnresolvedReferences
    import Tkinter as tkinter
except ImportError:
    # for Python3
    # noinspection PyPep8Naming,PyUnresolvedReferences
    import tkinter
try:
    # for Python2
    # noinspection PyPep8Naming,PyUnresolvedReferences
    import tkMessageBox as messagebox
except ImportError:
    # for Python3
    # noinspection PyPep8Naming,PyUnresolvedReferences,PyCompatibility
    from tkinter import messagebox

if not hasattr(sys, 'argv'):
    sys.argv = ['']


# ------------------------------------


class TopologyDialog(object):

    def __init__(self, parent, topology):
        top = self.top = tkinter.Toplevel(parent)
        top.wm_geometry("1000x150")
        tkinter.Label(top, text="Topology").pack()
        ent_topol = tkinter.Entry(top)
        ent_topol.insert(tkinter.END, topology)
        ent_topol.pack(padx=5, fill='both')
        but = tkinter.Button(top, text="OK", command=self.is_ok)
        but.pack(pady=5)

    def is_ok(self):
        self.top.destroy()


class GenerateTopologyGUI(object):

    def _add_gdcc(self):
        self.gdcc_frames.append(tkinter.Frame(self.root))
        self.gdcc_frames[-1].pack(side=tkinter.LEFT, fill='both')
        if not self.topology:
            max_gdcc = 0
        else:
            max_gdcc = max(key for key in self.topology)
        gdcc = max_gdcc + 1
        self.topology[gdcc] = {}
        self.asu_entries[gdcc] = {}
        self.dif_entries[gdcc] = {}
        # print "Add GDCC %d" %(gdcc)
        tkinter.Label(self.gdcc_frames[-1], text='GDCC %d' % gdcc).pack(fill='both')
        add_new_dif = partial(self._add_dif, gdcc)
        tkinter.Button(self.gdcc_frames[-1], text='<Add DIF>', command=add_new_dif).pack(fill='both')

    # ------------------------------------

    def _add_dif(self, gdcc):
        self.dif_frames.append([])
        self.dif_frames[gdcc - 1].append(tkinter.Frame(self.gdcc_frames[gdcc - 1]))
        self.dif_frames[gdcc - 1][-1].pack(side=tkinter.LEFT, fill='both')
        if not self.topology[gdcc]:
            max_dif = 0
        else:
            max_dif = max(key for key in self.topology[gdcc])
        dif = max_dif + 1
        self.topology[gdcc][dif] = {}
        self.asu_entries[gdcc][dif] = {}
        # print "GDCC %d : add DIF %d" %(gdcc, dif)
        dif_ent = tkinter.Entry(self.dif_frames[gdcc - 1][-1])
        dif_ent.insert(tkinter.END, 'DIF %d ID' % dif)
        tkinter.Label(self.dif_frames[gdcc - 1][-1], text='DIF %d' % dif).pack(fill='both')
        dif_ent.pack(fill='both')
        self.dif_entries[gdcc][dif] = dif_ent
        add_new_asu = partial(self._add_asu, gdcc, dif)
        tkinter.Button(self.dif_frames[gdcc - 1][-1], text='<Add ASU>',
                       command=add_new_asu).pack(fill='both')

    # ------------------------------------

    def _add_asu(self, gdcc, dif):
        ent = tkinter.Entry(self.dif_frames[gdcc - 1][dif - 1])
        ent.pack(fill='both')
        if not self.topology[gdcc][dif]:
            max_asu = 0
        else:
            max_asu = max(key for key in self.topology[gdcc][dif])
        asu = max_asu + 1
        self.topology[gdcc][dif][asu] = 0
        self.asu_entries[gdcc][dif][asu] = ent
        tkinter.Label(self.dif_frames[gdcc - 1][dif - 1], text='ASU %d' % asu).pack(fill='both')
        # print "GDCC %d : DIF %d : add ASU %d" %(gdcc, dif, asu)

    # ------------------------------------

    def _dif_id_is_sane(self):
        if not self.dif_entries:
            messagebox.showwarning("Warning", "DIF ID is empty")
            return False
        for gdcc, dif_entries_map in self.dif_entries.items():
            if not dif_entries_map:
                messagebox.showwarning("Warning", "DIF ID[GDCC %d] is empty" % gdcc)
                return False
            for dif, dif_entry in self.dif_entries[gdcc].items():
                dif_id = dif_entry.get()
                if dif is None or dif_id is None:
                    messagebox.showwarning("Warning", "DIF ID[GDCC %d] is empty" % gdcc)
                    return False
                try:
                    dif_id = int(dif_id)
                except ValueError:
                    messagebox.showwarning("Warning", "DIF ID[GDCC %d][DIF %s] is not an int"
                                           % (gdcc, dif))
                    return False
                self.dif_entries[gdcc][dif] = dif_id
        return True

    # ------------------------------------

    def _replace_dif_id(self):
        for gdcc in self.dif_entries:
            if self.topology[gdcc]:
                self.labeled_topology[gdcc] = {}
            for dif, dif_id in self.dif_entries[gdcc].items():
                if self.topology[gdcc][dif]:
                    self.labeled_topology[gdcc][dif_id] = {}
                    self.labeled_topology[gdcc][dif_id] = deepcopy(self.topology[gdcc][dif])

    # ------------------------------------

    def _topology_is_sane(self):
        if not self.topology:
            messagebox.showwarning("Warning", "topology is empty")
            return False
        for gdcc, dif_map in self.topology.items():
            if not dif_map:
                messagebox.showwarning("Warning", "topology[GDCC %d] is empty" % gdcc)
                return False
            for dif, asu_map in self.topology[gdcc].items():
                if not asu_map:
                    messagebox.showwarning("Warning", "topology[GDCC %d][DIF %d] is empty"
                                           % (gdcc, dif))
                    return False
                for asu, asu_entry in self.asu_entries[gdcc][dif].items():
                    n_channels = asu_entry.get()
                    if not n_channels or n_channels is None:
                        messagebox.showwarning("Warning", "topology[GDCC %d][DIF %d][ASU %d] "
                                                          "is empty" % (gdcc, dif, asu))
                        return False
                    self.topology[gdcc][dif][asu] = int(n_channels)
                    if int(n_channels) <= 0:
                        messagebox.showwarning("Warning", " number of channels "
                                                          "(topology[GDCC %d][DIF %d][ASU %d]) "
                                                          "is less or equal to zero" % (gdcc, dif, asu))
                        return False
        return True

    # ------------------------------------

    def _show_topology(self):
        if self._topology_is_sane():
            if self._dif_id_is_sane():
                self._replace_dif_id()
                self.topology.clear()
                self.topology = self.labeled_topology.copy()
            else:
                return False
            self.topology_json = json.dumps(self.topology)
            dialog = TopologyDialog(self.root, self.topology_json)
            self.show_topology_button["state"] = "disabled"
            self.root.wait_window(dialog.top)
            self.show_topology_button["state"] = "normal"
            self.root.quit()
            return True
        else:
            return False

    # ------------------------------------

    def __init__(self, persistent_mode=None):
        self.gdcc_frames = []
        self.dif_frames = []
        self.topology = {}
        self.labeled_topology = {}
        self.asu_entries = {}
        self.dif_entries = {}
        self.topology_json = None
        self.root = None
        self.show_topology_button = None
        self.persistent_mode = persistent_mode

    # ------------------------------------

    def run(self):
        self.root = tkinter.Tk()
        self.root.title("WAGASCI topology generator")
        self.root.option_add("*Dialog.msg.wrapLength", "10i")

        show_button = self.show_topology_button = tkinter.Button(self.root, text='Print topology string',
                                                                 command=self._show_topology)
        show_button.pack(fill='both')

        add_gdcc_button = tkinter.Button(self.root, text='<Add GDCC>', fg="Blue", command=self._add_gdcc)
        add_gdcc_button.pack(fill='both')

        try:
            self.root.mainloop()
            if self.persistent_mode is None:
                self.root.destroy()
            return_json = self.topology_json
            return return_json
        except AttributeError:
            self.root.destroy()
            return ""


# ------------------------------------

def topology_generator(persistent_mode=None):
    gui = GenerateTopologyGUI(persistent_mode)
    return gui.run()


# ------------------------------------

if __name__ == "__main__":
    topology_generator("persistent_mode")



import unittest

from wagascianpy.pyrmod.CCC import CCC


class TestCCC(unittest.TestCase):

    def setUp(self):
        self.spill_period = 100
        self.active_time = 200
        self.simulation_dic = {
            "get_spill_type": (1, "undefined"),
            "get_spill_period": (1, self.spill_period),
            "get_active_time": (1, self.active_time)
        }
        self.ccc = CCC(simulate=True, simulation_dic=self.simulation_dic)

    def test_undefined(self):
        self.ccc.simulation_dic["get_spill_type"] = (1, "undefined")
        self.assertEqual(self.ccc.simulation_dic["get_spill_type"],
                         self.ccc.get_spill_type())

    def test_spill_off(self):
        self.ccc.simulation_dic["get_spill_type"] = (1, "spill off")
        self.ccc.spill_off()
        self.assertEqual(self.ccc.simulation_dic["get_spill_type"],
                         self.ccc.get_spill_type())

    def test_continuous(self):
        self.ccc.simulation_dic["get_spill_type"] = (1, "continuous")
        self.ccc.spill_continuous()
        self.assertEqual(self.ccc.simulation_dic["get_spill_type"],
                         self.ccc.get_spill_type())

    def test_external_spill(self):
        self.ccc.simulation_dic["get_spill_type"] = (1, "external spill")
        self.ccc.spill_external()
        self.assertEqual(self.ccc.simulation_dic["get_spill_type"],
                         self.ccc.get_spill_type())

    def test_internal_spill(self):
        self.ccc.simulation_dic["get_spill_type"] = (1, "internal spill")
        self.ccc.spill_internal()
        self.assertEqual(self.ccc.simulation_dic["get_spill_type"],
                         self.ccc.get_spill_type())
        self.assertEqual(self.ccc.simulation_dic["get_spill_period"],
                         self.ccc.get_spill_period())
        self.assertEqual(self.ccc.simulation_dic["get_active_time"],
                         self.ccc.get_active_time())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2019 Pintaudi Giorgio

""" Run script to measure and check the pedestal of the SPIROC2D ASIC """

# pylint: disable-msg=too-many-arguments
# pylint: disable-msg=too-many-function-args
# pylint: disable-msg=too-many-locals

import argparse
# Python modules
import json
import os
import re
import textwrap

from bitarray import bitarray

# WAGASCI modules
from wagascianpy.analysis.analysis import WagasciAnalysis
from wagascianpy.utils.environment import WagasciEnvironment
from wagascianpy.utils.utils import limit_threads, join_threads

RANGE_PEU = [1, 2]
MAX_NB_THREAD_CHAINS = 8


# ================================================================ #
#                                                                  #
#                        Analysis functions                        #
#                                                                  #
# ================================================================ #

def pedestal_decoder(process, idac_dir, acq_name, peu, threshold_dir,
                     dif, n_chips):
    """ wgDecoder for the pedestal_run.py script """
    peu = int(peu)
    dif = int(dif)
    n_chips = int(n_chips)
    input_raw_file = "%s/PEU%s/%s/RawData/%s_ecal_dif_%s.raw" \
                     % (idac_dir, peu, threshold_dir, acq_name, dif)
    output_dir = "%s/PEU%s/%s/wgDecoder" % (idac_dir, peu, threshold_dir)
    overwrite_flag = True
    compatibility_mode = False
    return process.spawn("decoder", input_raw_file, "", output_dir,
                         overwrite_flag, compatibility_mode, dif, n_chips)


def pedestal_make_hist(process, idac_dir, acq_name, peu, threshold_dir,
                       dif, acq_config_xml):
    """ wgMakeHist for the pedestal_run.py script """
    peu = int(peu)
    dif = int(dif)
    input_tree_file = "%s/PEU%s/%s/wgDecoder/%s_ecal_dif_%s_tree.root" \
                      % (idac_dir, peu, threshold_dir, acq_name, dif)
    output_dir = "%s/PEU%s/%s/wgMakeHist" % (idac_dir, peu, threshold_dir)
    flags = bitarray('0' * 9, endian='big')
    flags[7] = True  # dark noise
    flags[4] = True  # charge hit HG
    flags[0] = True  # overwrite
    ul_flags = int(flags.to01(), 2)
    return process.make_hist(input_tree_file, acq_config_xml,
                             output_dir, ul_flags, dif)


def pedestal_ana_hist(process, idac_dir, acq_name, peu, threshold_dir,
                      dif, acq_config_xml):
    """ wgAnaHist for the pedestal_run.py script """
    peu = int(peu)
    dif = int(dif)
    input_hist_file = "%s/PEU%s/%s/wgMakeHist/%s_ecal_dif_%s_hist.root" \
                      % (idac_dir, peu, threshold_dir, acq_name, dif)
    output_dir = "%s/PEU%s/%s/wgAnaHist/Xml/dif%s" \
                 % (idac_dir, peu, threshold_dir, dif)
    output_img_dir = "%s/PEU%s/%s/wgAnaHist/Images/dif%s" \
                     % (idac_dir, peu, threshold_dir, dif)
    flags = bitarray('0' * 8, endian='big')
    flags[7] = True  # overwrite
    flags[6] = bool(acq_config_xml)
    flags[5] = False  # print
    flags[4] = True  # Dark noise
    flags[3] = True  # charge nohit
    flags[2] = True  # charge hit HG
    ul_flags = int(flags.to01(), 2)
    return process.spawn("ana_hist", input_hist_file, acq_config_xml,
                         output_dir, output_img_dir, ul_flags, dif)


def pedestal_ana_hist_summary(process, idac_dir, peu, threshold_dir, dif):
    """ wgAnaHistSummary for the pedestal_run.py script """
    peu = int(peu)
    dif = int(dif)
    input_dir = "%s/PEU%s/%s/wgAnaHist/Xml/dif%s" \
                % (idac_dir, peu, threshold_dir, dif)
    output_dir = "%s/PEU%s/%s/wgAnaHistSummary/Xml/dif%s" \
                 % (idac_dir, peu, threshold_dir, dif)
    output_img_dir = "%s/PEU%s/%s/wgAnaHistSummary/Images/dif%s" \
                     % (idac_dir, peu, threshold_dir, dif)
    flags = bitarray('0' * 8, endian='big')
    flags[7] = True  # overwrite
    flags[5] = False  # print
    flags[4] = True  # Dark noise
    flags[3] = True  # charge nohit
    flags[2] = True  # charge hit HG
    ul_flags = int(flags.to01(), 2)
    return process.ana_hist_summary(input_dir, output_dir, output_img_dir,
                                    ul_flags)


def pedestal_pedestal_calib(process, idac_dir):
    """ wgPedestalCalib for the pedestal_run.py script """
    output_dir = idac_dir + "/wgPedestalCalib/Xml"
    output_img_dir = idac_dir + "/wgPedestalCalib/Images"
    return process.pedestal_calib(idac_dir, output_dir, output_img_dir)


def _get_idac_dir(run_root_dir):
    dir_list = [name for name in os.listdir(run_root_dir)
                if os.path.isdir(os.path.join(run_root_dir, name))]
    for name in dir_list:
        if len(re.findall(r'\d+', name)) == 1:
            return int(re.findall(r'\d+', name)[0])
        if "OPTIMIZE" in name:
            return "OPTIMIZE"
    return None


def pedestal_analysis(run, decoder=True, makehist=True, anahist=True, overwrite=False, single_threshold=False):
    """ Analyze pedestal calibration data """

    decoder = bool(decoder)
    makehist = bool(makehist)
    anahist = bool(anahist)
    overwrite = bool(overwrite)
    single_threshold = bool(single_threshold)

    # Environmental variables
    env = WagasciEnvironment()
    wagasci_lib = env["WAGASCI_LIBDIR"]

    if not os.path.exists(run):
        run = os.path.join(env['WAGASCI_CALIBDATADIR'], run)

    run_name = os.path.basename(run)
    run_root_dir = run

    input_dac = _get_idac_dir(run_root_dir)
    if input_dac is None:
        raise ValueError("Input DAC not recognized")
    idac_dir = os.path.join(run_root_dir, "iDAC%s" % input_dac)
    acq_config_base = os.path.basename(env['WAGASCI_ACQCONFIGDIR'])
    acq_config_xml = os.path.join(run_root_dir, acq_config_base, env['WAGASCI_ACQCONFIGXML'])
    if not os.path.exists(acq_config_xml):
        print("Acquisition configuration XML file not found : %s" % acq_config_xml)
    if single_threshold:
        threshold_dirs = ["SingleTh"]
    else:
        threshold_dirs = ["LowTh", "MiddleTh", "HighTh"]

    # =========================================================== #
    #                        ANALYZE DATA                         #
    # =========================================================== #

    process = WagasciAnalysis(wagasci_lib)
    dif_topology_string, dif_topology_pointer = process.get_dif_topology(acq_config_xml)
    dif_topology = json.loads(dif_topology_string)
    process.free_topology(dif_topology_pointer)
    process.enable_thread_safety()
    del process

    ###########################################################################
    #                                 decoder                                 #
    ###########################################################################

    if decoder:

        threads = []

        for peu in RANGE_PEU:
            for threshold_dir in threshold_dirs:
                acq_name = "%s_iDAC%s_PEU%s_%s" % (run_name, input_dac, peu, threshold_dir)
                for dif in dif_topology:
                    decoded_filename = os.path.join(idac_dir, "PEU%s" % peu, threshold_dir, "wgDecoder",
                                                    "%s_ecal_dif_%s_tree.root" % (acq_name, dif))
                    if not overwrite and os.path.exists(decoded_filename):
                        print("Skipping %s" % decoded_filename)
                        continue
                    limit_threads(threads, MAX_NB_THREAD_CHAINS)
                    process = WagasciAnalysis(wagasci_lib)
                    n_chips = len(dif_topology[dif])
                    threads.append(pedestal_decoder(process, idac_dir, acq_name, peu, threshold_dir, dif, n_chips))
                    del process
                # dif loop
            # threshold loop
        # peu loop

        # Wait until all the threads have returned
        join_threads(threads)

    ###########################################################################
    #                                make_hist                                #
    ###########################################################################

    if makehist:

        for peu in RANGE_PEU:
            for threshold_dir in threshold_dirs:
                acq_name = "%s_iDAC%s_PEU%s_%s" % (run_name, input_dac, peu, threshold_dir)
                actual_acq_config_xml = os.path.join(idac_dir, "PEU%s" % peu, threshold_dir, "RawData",
                                                     "%s.xml" % acq_name)
                for dif in dif_topology:
                    makehist_filename = os.path.join(idac_dir, "PEU%s" % peu, threshold_dir,  "wgMakeHist",
                                                     "%s_ecal_dif_%s_hist.root" % (acq_name, dif))
                    if not overwrite and os.path.exists(makehist_filename):
                        print("Skipping %s" % makehist_filename)
                        continue
                    process = WagasciAnalysis(wagasci_lib)
                    result = pedestal_make_hist(process, idac_dir, acq_name, peu,
                                                threshold_dir, dif, actual_acq_config_xml)
                    if result != 0:
                        print("wgMakeHist returned error code %s" % result)
                        exit(result)
                    del process
                # dif loop
            # threshold loop
        # peu loop

    ###########################################################################
    #                                 ana_hist                                #
    ###########################################################################

    if anahist:

        threads = []

        for peu in RANGE_PEU:
            for threshold_dir in threshold_dirs:
                acq_name = "%s_iDAC%s_PEU%s_%s" % (run_name, input_dac, peu, threshold_dir)
                actual_acq_config_xml = os.path.join(idac_dir, "PEU%s" % peu, threshold_dir, "RawData",
                                                     "%s.xml" % acq_name)

                for dif in dif_topology:
                    anahist_dirname = os.path.join(idac_dir, "PEU%s" % peu, threshold_dir, "wgAnaHist", "Xml",
                                                   "dif%s" % dif)
                    if not overwrite and os.path.exists(anahist_dirname):
                        print("Skipping %s" % anahist_dirname)
                        continue
                    limit_threads(threads, MAX_NB_THREAD_CHAINS)
                    process = WagasciAnalysis(wagasci_lib)
                    threads.append(pedestal_ana_hist(process, idac_dir, acq_name, peu,
                                                     threshold_dir, dif, actual_acq_config_xml))
                    # result = pedestal_ana_hist(process, idac_dir, acq_name, peu,
                    #                            dif, actual_acq_config_xml)
                    # if result != 0:
                    #     print "wgAnaHist returned error code %s" %(result)
                    #     exit(result)
                    del process
                # dif loop
            # threshold dir
        # peu loop

        # Wait until all the threads have returned
        join_threads(threads)

    ###########################################################################
    #                             ana_hist_summary                            #
    ###########################################################################

    if anahist:

        for peu in RANGE_PEU:
            for threshold_dir in threshold_dirs:
                for dif in dif_topology:
                    anahist_dirname = os.path.join(idac_dir, "PEU%s" % peu, threshold_dir, "wgAnaHistSummary", "Xml",
                                                   "dif%s" % dif)
                    if not overwrite and os.path.exists(anahist_dirname):
                        print("Skipping %s" % anahist_dirname)
                        continue
                    process = WagasciAnalysis(wagasci_lib)
                    result = pedestal_ana_hist_summary(process, idac_dir, peu, threshold_dir, dif)
                    if result != 0:
                        print("wgAnaHistSummary returned error code %s" % result)
                        exit(result)
                    del process
                # dif loop
            # threshold loop
        # peu loop

    process = WagasciAnalysis(wagasci_lib)
    result = pedestal_pedestal_calib(process, idac_dir)
    del process
    if result != 0:
        print("wgPedestalCalib returned error code %s" % result)
        exit(result)


if __name__ == "__main__":
    PARSER = argparse.ArgumentParser(usage='use "python %(prog)s --help" for more information',
                                     argument_default=None, description='Analyze the pedestal calibration data')

    PARSER.add_argument('-f', '--run', metavar='<run name or path>', type=str, required=True,
                        help='Name or path of the run to analyze. If only the run name is given the run must be inside '
                             'the ${WAGASCI_CALIBDATADIR} folder.')
    PARSER.add_argument('-d', '--disable-decoder', dest='decoder', action='store_false',
                        required=False, default=True, help="Disable the wgDecoder.")
    PARSER.add_argument('-m', '--disable-makehist', dest='makehist', action='store_false',
                        required=False, default=True, help="Disable the wgMakeHist.")
    PARSER.add_argument('-a', '--disable-anahist', dest='anahist', action='store_false',
                        required=False, default=True, help="Disable the wgAnaHist and wgAnaHistSummary.")
    PARSER.add_argument('-r', '--overwrite', dest='overwrite', action='store_true',
                        required=False, default=False, help="Overwrite flag.")
    PARSER.add_argument('-s', '--single-threshold', dest='single_threshold', action='store_true',
                        required=False, default=False, help=textwrap.dedent('''
                        If not set it is assumed that the low threshold LowTh, high threshold HighTh and middle
                        threshold MiddleTh folders are present. If set only the SingleTh folder is analyzed.
                        '''))

    ARGS = PARSER.parse_args()

    RUN = ARGS.run
    DECODER = ARGS.decoder
    MAKEHIST = ARGS.makehist
    ANAHIST = ARGS.anahist
    OVERWRITE = ARGS.overwrite
    SINGLE_THRESHOLD = ARGS.single_threshold

    pedestal_analysis(run=RUN, decoder=DECODER, makehist=MAKEHIST, anahist=ANAHIST,
                      overwrite=OVERWRITE, single_threshold=SINGLE_THRESHOLD)
